import os

__version__ = '1.2.0'
__author__ = 'Ryo Kurosawa'
__license__      = 'MIT'
__author_email__ = 'a0160561@yahoo.co.jp'
__url__          = 'https://github.com/shiro-kur/PDIVAS'

__cur_path__ = __file__

